﻿// BalloonPlus.js Ver.1.1.0
// MIT License (C) 2023 あわやまたな
// http://opensource.org/licenses/mit-license.php

/*:
* @target MZ
* @plugindesc I summarized the functions I want for the balloon icon.
* @author あわやまたな (Awaya_Matana)
* @url https://awaya3ji.seesaa.net/
* @help Initial parameters are for balloons of RPG Maker DS+ Material Collection for MV.

* If you use other materials, please read the explanation of the parameters and adjust them.
*
* The parameters are also reflected in the event command ［フキダシアイコンの表示］,

* Requires the use of plugin commands for full functionality.
*
* [スクリプト（移動ルートの設定）]
* 【フキダシアイコンの表示】
* this.showBalloon(balloonId, wait, loop, noSe);
*
* balloonId：フキダシIDを入力します。
* びっくり：1　はてな：2　音符：3…とBalloon.pngの上から連番になります。
* wait：ウェイトするかをtrue/falseで入力します。未入力だとウェイトしません。
* loop：ループするかをtrue/falseで入力します。未入力だとパラメータ設定を使用。
* noSe：効果音を再生しないかをtrue/falseで入力します。未入力だとfalse。
*
* 【フキダシアイコンの消去】
* this.removeBalloon();
*
* (入力例)
* this.showBalloon(4); //Show hearts.
* this.showBalloon(4, true); //Show a heart and wait until it finishes.
* this.showBalloon(1, false, true); //Surprise loop.
* this.showBalloon(2, false, false, true); //Shows the welcome without sound effects.
* this.removeBalloon(); //erase.
*
* [注釈]
* Place a comment on the first line of the event execution content and enter it in the following format to reflect it.
* 【シンボルバルーン】
* <symbolBalloon:balloonId,keep>
*
* balloonId：フキダシIDを入力します。
* keep：イベント起動後も表示し続けるかをtrue/falseで入力します。未入力だとfalse。
*
* (入力例)
* <symbolBalloon:1> //Always display a surprise over the head of the event.
* Disappears only when the event is running.
* <symbolBalloon:2,true> //Always displays a question mark above the head of the event.
* It will continue to be displayed even when the event is running.
*
* [名前リスト]
* Since it is difficult to grasp the balloon ID, you can enter this instead of the ID.
*
* (入力例)
* this.showBalloon("ハート");
* this.showBalloon("ハート", true);
* this.showBalloon("びっくり", false, true);
* this.showBalloon("はてな", false, false, true);
* <symbolBalloon:びっくり>
* <symbolBalloon:はてな,true>
*
* [更新履歴]
* 2023/01/02：Ver.1.0.0　公開。
* 2023/01/25：Ver.1.0.1　プラグインコマンドのウェイトを修正。
* 2023/01/25：Ver.1.1.0　IDだけでなく、文字列でも呼び出し可能に。
*
* @param offsetX
* @text オフセットX
* @desc Distance to shift the balloon.
* @type number
* @default 16
*
* @param offsetY
* @text オフセットY
* @desc Distance to shift the balloon.
* @type number
* @default 0
*
* @param speed
* @text 速度
* @desc Time until the balloon transitions to the next frame.
* MZ標準は８。
* @type number
* @default 6
*
* @param waitTime
* @text ウェイト時間
* @desc Remaining time after balloon playback ends.
* MZ standard is 12。
* @type number
* @default 0
*
* @param muteSwitchId
* @text ミュートスイッチ
* @desc While this switch is on, balloon sound effects will not play.
* If not specified, always play.
* @type switch
*
* @param balloonSettings
* @text バルーン設定
* @type struct<balloonSetting>[]
* @default ["{\"id\":\"1\",\"se\":\"{\\\"name\\\":\\\"Balloon11\\\",\\\"volume\\\":\\\"90\\\",\\\"pitch\\\":\\\"100\\\",\\\"pan\\\":\\\"0\\\"}\",\"offsetX\":\"0\",\"offsetY\":\"0\",\"startIndex\":\"0\",\"numFrames\":\"8\",\"loopEnabled\":\"false\"}","{\"id\":\"2\",\"se\":\"{\\\"name\\\":\\\"Balloon2\\\",\\\"volume\\\":\\\"90\\\",\\\"pitch\\\":\\\"100\\\",\\\"pan\\\":\\\"0\\\"}\",\"offsetX\":\"0\",\"offsetY\":\"0\",\"startIndex\":\"0\",\"numFrames\":\"8\",\"loopEnabled\":\"false\"}","{\"id\":\"3\",\"se\":\"{\\\"name\\\":\\\"Balloon6\\\",\\\"volume\\\":\\\"90\\\",\\\"pitch\\\":\\\"100\\\",\\\"pan\\\":\\\"0\\\"}\",\"offsetX\":\"0\",\"offsetY\":\"0\",\"startIndex\":\"0\",\"numFrames\":\"8\",\"loopEnabled\":\"false\"}","{\"id\":\"4\",\"se\":\"{\\\"name\\\":\\\"Balloon6\\\",\\\"volume\\\":\\\"90\\\",\\\"pitch\\\":\\\"100\\\",\\\"pan\\\":\\\"0\\\"}\",\"offsetX\":\"0\",\"offsetY\":\"0\",\"startIndex\":\"0\",\"numFrames\":\"8\",\"loopEnabled\":\"false\"}","{\"id\":\"5\",\"se\":\"{\\\"name\\\":\\\"Balloon1\\\",\\\"volume\\\":\\\"90\\\",\\\"pitch\\\":\\\"100\\\",\\\"pan\\\":\\\"0\\\"}\",\"offsetX\":\"0\",\"offsetY\":\"0\",\"startIndex\":\"0\",\"numFrames\":\"8\",\"loopEnabled\":\"false\"}","{\"id\":\"6\",\"se\":\"{\\\"name\\\":\\\"Balloon5\\\",\\\"volume\\\":\\\"90\\\",\\\"pitch\\\":\\\"100\\\",\\\"pan\\\":\\\"0\\\"}\",\"offsetX\":\"0\",\"offsetY\":\"0\",\"startIndex\":\"0\",\"numFrames\":\"8\",\"loopEnabled\":\"false\"}","{\"id\":\"7\",\"se\":\"{\\\"name\\\":\\\"Balloon10\\\",\\\"volume\\\":\\\"90\\\",\\\"pitch\\\":\\\"100\\\",\\\"pan\\\":\\\"0\\\"}\",\"offsetX\":\"0\",\"offsetY\":\"0\",\"startIndex\":\"0\",\"numFrames\":\"8\",\"loopEnabled\":\"false\"}","{\"id\":\"9\",\"se\":\"{\\\"name\\\":\\\"Balloon4\\\",\\\"volume\\\":\\\"90\\\",\\\"pitch\\\":\\\"100\\\",\\\"pan\\\":\\\"0\\\"}\",\"offsetX\":\"0\",\"offsetY\":\"0\",\"startIndex\":\"0\",\"numFrames\":\"8\",\"loopEnabled\":\"false\"}","{\"id\":\"10\",\"se\":\"{\\\"name\\\":\\\"Sleep\\\",\\\"volume\\\":\\\"90\\\",\\\"pitch\\\":\\\"100\\\",\\\"pan\\\":\\\"0\\\"}\",\"offsetX\":\"0\",\"offsetY\":\"0\",\"startIndex\":\"0\",\"numFrames\":\"8\",\"loopEnabled\":\"false\"}","{\"id\":\"11\",\"se\":\"{\\\"name\\\":\\\"Balloon11\\\",\\\"volume\\\":\\\"90\\\",\\\"pitch\\\":\\\"100\\\",\\\"pan\\\":\\\"0\\\"}\",\"offsetX\":\"0\",\"offsetY\":\"8\",\"startIndex\":\"1\",\"numFrames\":\"4\",\"loopEnabled\":\"false\"}","{\"id\":\"12\",\"se\":\"\",\"offsetX\":\"0\",\"offsetY\":\"8\",\"startIndex\":\"1\",\"numFrames\":\"3\",\"loopEnabled\":\"true\"}","{\"id\":\"13\",\"se\":\"\",\"offsetX\":\"0\",\"offsetY\":\"8\",\"startIndex\":\"1\",\"numFrames\":\"4\",\"loopEnabled\":\"true\"}","{\"id\":\"14\",\"se\":\"\",\"offsetX\":\"0\",\"offsetY\":\"8\",\"startIndex\":\"1\",\"numFrames\":\"5\",\"loopEnabled\":\"true\"}","{\"id\":\"15\",\"se\":\"\",\"offsetX\":\"0\",\"offsetY\":\"8\",\"startIndex\":\"1\",\"numFrames\":\"5\",\"loopEnabled\":\"false\"}","{\"id\":\"16\",\"se\":\"{\\\"name\\\":\\\"Balloon8\\\",\\\"volume\\\":\\\"90\\\",\\\"pitch\\\":\\\"100\\\",\\\"pan\\\":\\\"0\\\"}\",\"offsetX\":\"0\",\"offsetY\":\"8\",\"startIndex\":\"1\",\"numFrames\":\"4\",\"loopEnabled\":\"true\"}"]
*
* @param nameList
* @text 名前リスト
* @desc Set a name for each balloon ID.
* It is also possible to use this instead of the balloon ID.
* @type string[]
* @default ["びっくり","はてな","音符","ハート","怒り","汗","くしゃくしゃ","沈黙","電球","Zzz","びっくり２","笑い","怒り２","汗２","気づく","ショック"]
*
* @command showBalloon
* @text フキダシアイコンの表示
* @desc Displays a balloon icon.

* @arg characterId
* @text キャラクターID
* @desc Player: -1 This event: 0
* @default 0
*
* @arg balloonId
* @text フキダシID
* @desc びっくり：1　はてな：2　音符：3　ハート：4　怒り：5
* 汗：6　くしゃくしゃ：7　沈黙：8　電球：9　Zzz：10
* @default びっくり
* @type combo
* @option びっくり
* @option はてな
* @option 音符
* @option ハート
* @option 怒り
* @option 汗
* @option くしゃくしゃ
* @option 沈黙
* @option 電球
* @option Zzz
* @option びっくり２
* @option 笑い
* @option 怒り２
* @option 汗２
* @option 気づく
* @option ショック
*
* @arg loopEnabled
* @text ループ有効化
* @desc It will continue to display until you stop it.
* If it is blank, the parameter setting will be used.
* @type boolean
*
* @arg noSe
* @text 効果音なし
* @desc Does not play the sound effect set by the parameter.
* @type boolean
* @default false
*
* @arg wait
* @text 完了までウェイト
* @desc Wait for the effect to finish.
* @type boolean
* @default false
*
* @command removeBalloon
* @text フキダシアイコンの消去
* @desc Clears the balloon icon.

* @arg characterId
* @text キャラクターID
* @desc Player: -1 This event: 0
* ReplaceTargetEvent.jsAt the time of introduction, platoon members can also be specified.
* @default 0
*
*/

/*~struct~balloonSetting:
*
* @param id
* @text フキダシID
* @desc びっくり：1　はてな：2　音符：3　ハート：4　怒り：5
* 汗：6　くしゃくしゃ：7　沈黙：8　電球：9　Zzz：10
* @type number
* @default 1
* @min 1
*
* @param se
* @text 効果音
* @desc This is the sound effect that sounds when the balloon is displayed.
* Symbols do not ring.
* @type struct<audio>
*
* @param offsetX
* @text オフセットX
* @desc Distance to shift the balloon.
* @type number
* @default 0
*
* @param offsetY
* @text オフセットY
* @desc Distance to shift the balloon.
* @type number
* @default 0
*
* @param startIndex
* @text 開始インデックス
* @desc The animation start position of the balloon.
* The left edge of Balloon.png is 0 and the right edge is 8.
* @type number
* @default 0
* 
* @param numFrames
* @text コマ数
* @desc The number of frames used for animation.
* @type number
* @default 8
*
* @param loopEnabled
* @text ループ有効化
* @desc It will continue to display until you stop it.
* @type boolean
* @default false
*
*/

/*~struct~audio:
*
* @param name
* @text 名前
* @type file
* @dir audio/se
*
* @param volume
* @text 音量
* @type number
* @default 90
* @min 0
*
* @param pitch
* @text ピッチ
* @type number
* @default 100
*
* @param pan
* @text 位相
* @type number
* @default 0
* @min -100
*
*/

'use strict';
{
	const useMZ = Utils.RPGMAKER_NAME === "MZ";
	const pluginName = document.currentScript.src.match(/^.*\/(.*).js$/)[1];
	const hasPluginCommonBase = typeof PluginManagerEx === "function";
	const parameters = PluginManager.parameters(pluginName);

	const offsetX = Number(parameters["offsetX"]);
	const offsetY = Number(parameters["offsetY"]);
	const speed = Number(parameters["speed"]);
	const waitTime = Number(parameters["waitTime"]);
	const nameList = JSON.parse(parameters["nameList"] || "[]");
	const muteSwitchId = Number(parameters["muteSwitchId"] || 0);

	const balloonSettings = [];
	JSON.parse(parameters["balloonSettings"]).forEach(str => {
		const params = analyticsParameters(str);
		balloonSettings[params.id] = params;
	});

	function analyticsParameters(str) {
		const params = str ? JSON.parse(str) : {};
		for (const prop in params) {
			const param = params[prop];
			if (isFinite(param) || param === 'true' || param === 'false' || (typeof param === "string" && param.startsWith("{") && param.endsWith("}"))) {
				params[prop] = analyticsParameters(param);
			}
		}
		return params;
	}

	function findBalloonId(str) {
		const index = nameList.findIndex(n => n === str);
		if (index > -1) {
			return index + 1;
		}
		return Number(str);
	};

	//-----------------------------------------------------------------------------
	// PluginManager

	PluginManager.registerCommand(pluginName, "showBalloon", function (args) {
		this._characterId = +args.characterId;
		const character = this.character(this._characterId);
		if (character) {
			const balloonId = findBalloonId(args.balloonId);
			const loopEnabled = args.loopEnabled === "" ? null : args.loopEnabled === "true";
			const noSe = args.noSe === "true";
			this._characterId = characterId;
			character.showBalloon(balloonId, false, loopEnabled, noSe);
			if (args.wait === "true") {
				this.setWaitMode("balloon");
			}
		}
	});

	PluginManager.registerCommand(pluginName, "removeBalloon", function (args) {
		const character = this.character(+args.characterId);
		if (character) {
			character.removeBalloon();
		}
	});

	if (hasPluginCommonBase) {
		PluginManagerEx.registerCommand(document.currentScript, "showBalloon", function (args) {
			this._characterId = args.characterId;
			const character = this.character(this._characterId);
			if (character) {
				const balloonId = findBalloonId(args.balloonId);
				const loopEnabled = args.loopEnabled === "" ? null : args.loopEnabled;
				const noSe = args.noSe;
				character.showBalloon(balloonId, false, loopEnabled, noSe);
				if (args.wait) {
					this.setWaitMode("balloon");
				}
			}
		});
	}

	//-----------------------------------------------------------------------------
	// Game_Temp

	const _Game_Temp_requestBalloon = Game_Temp.prototype.requestBalloon;
	Game_Temp.prototype.requestBalloon = function(target, balloonId) {
		_Game_Temp_requestBalloon.apply(this, arguments);
		const request = this._balloonQueue[this._balloonQueue.length - 1];
		if (target.setBalloonId) {
			target.setBalloonId(balloonId);
		}
		if (target.setLoopingBalloon) {
			const setting = balloonSettings[balloonId];
			const loopEnabled = !!setting && setting.loopEnabled;
			target.setLoopingBalloon(loopEnabled);
			request.loop = loopEnabled;
		}
	};

	Game_Temp.prototype.requestBalloonPlus = function(target, balloonId, loop = null, noSe) {
		Game_Temp.prototype.requestBalloon.call(this, target, balloonId);
		const request = this._balloonQueue[this._balloonQueue.length - 1];
		request.noSe = noSe;
		if (loop !== null && target.setLoopingBalloon) {
			target.setLoopingBalloon(loop);
			request.loop = loop;
		}
	};

	//-----------------------------------------------------------------------------
	// Game_CharacterBase

	const _Game_CharacterBase_initMembers = Game_CharacterBase.prototype.initMembers;
	Game_CharacterBase.prototype.initMembers = function() {
		_Game_CharacterBase_initMembers.call(this);
		this._balloonId = 0;
		this._loopingBalloon = false;
		this._symbolBalloonId = 0;
	};

	Game_CharacterBase.prototype.setBalloonId = function(balloonId) {
		this._balloonId = balloonId;
	};

	Game_CharacterBase.prototype.balloonId = function() {
		return this._balloonId;
	};

	Game_CharacterBase.prototype.setSymbolBalloonId = function(balloonId) {
		this._symbolBalloonId = balloonId;
	};

	Game_CharacterBase.prototype.symbolBalloonId = function() {
		return this._symbolBalloonId;
	};

	Game_CharacterBase.prototype.setLoopingBalloon = function(loop) {
		this._loopingBalloon = loop;
	};

	Game_CharacterBase.prototype.loopingBalloon = function() {
		return this._loopingBalloon;
	};

	Game_CharacterBase.prototype.showBalloon = function(balloonId, wait, loop = null, noSe) {
		balloonId = findBalloonId(balloonId);
		$gameTemp.requestBalloonPlus(this, balloonId, loop, noSe);
		if (wait) {
			const setting = balloonSettings[balloonId] || {};
			const numFrames = setting.numFrames ?? 8;
			this._waitCount = numFrames * speed + waitTime;
		}
	};

	Game_CharacterBase.prototype.removeBalloon = function() {
		$gameTemp.requestBalloon(this, 0);
	};

	//-----------------------------------------------------------------------------
	// Sprite_Character

	const _Sprite_Character_initialize = Sprite_Character.prototype.initialize;
	Sprite_Character.prototype.initialize = function(character) {
		_Sprite_Character_initialize.call(this, character);
		this.restoreBalloon();
	};

	Sprite_Character.prototype.restoreBalloon = function() {
		const character = this._character;
		const balloonId = character.balloonId ? character.balloonId() : 0;
		const loopEnabled = character.loopingBalloon && character.loopingBalloon();
		if (balloonId > 0 && character.showBalloon) {
			character.showBalloon(balloonId, false, loopEnabled, true);
		}
	};

	//-----------------------------------------------------------------------------
	// Sprite_Balloon

	const _Sprite_Balloon_initMembers = Sprite_Balloon.prototype.initMembers;
	Sprite_Balloon.prototype.initMembers = function() {
		_Sprite_Balloon_initMembers.call(this);
		this._offsetX = 0;
		this._offsetY = 0;
		this._startIndex = 0;
		this._numFrames = 0;
		this._loopEnabled = false;
		this._request = null;
	}

	const _Sprite_Balloon_setup = Sprite_Balloon.prototype.setup;
	Sprite_Balloon.prototype.setup = function(targetSprite, balloonId) {
		_Sprite_Balloon_setup.apply(this, arguments);
		this._offsetX = 0;
		this._offsetY = 0;
		this._startIndex = 0;
		this._numFrames = 8;
		this._loopEnabled = false;
		this._request = null;
	};

	Sprite_Balloon.prototype.setupBalloonPlus = function(request) {
		const targetSprite = this._target;
		const balloonId = this._balloonId;
		const targetObject = this.targetObject;
		const setting = balloonSettings[balloonId] || {};
		const seParam = setting.se;
		this._request = request;
		this._offsetX = offsetX;
		this._offsetY = offsetY;
		this._offsetX += setting.offsetX ?? 0;
		this._offsetY += setting.offsetY ?? 0;
		this._startIndex = setting.startIndex ?? 0;
		this._numFrames = setting.numFrames ?? 8;
		this._loopEnabled = request.loop;
		if (!request.noSe && !$gameSwitches.value(muteSwitchId) && seParam) {
			AudioManager.playSe(seParam);
		}
		this.resetDuration();
	};

	Sprite_Balloon.prototype.resetDuration = function() {
		if (this._balloonId === 0) {
			this._duration = 0;
		} else {
			this._duration = this.numFrames() * this.speed() + this.waitTime();
		}
	};

	const _Sprite_Balloon_update = Sprite_Balloon.prototype.update;
	Sprite_Balloon.prototype.update = function() {
		_Sprite_Balloon_update.call(this);
		if (this.loopEnabled() && this._duration === this.waitTime()) {
			this.resetDuration();
			this.update();
		}
	};

	Sprite_Balloon.prototype.frameIndex = function() {
		const index = (this._duration - this.waitTime()) / this.speed();
		return (this.numFrames() - 1) - Math.max(Math.floor(index), 0) + this.startIndex();
	};

	Sprite_Balloon.prototype.startIndex = function() {
		return this._startIndex;
	};

	Sprite_Balloon.prototype.numFrames = function() {
		return this._numFrames;
	};

	Sprite_Balloon.prototype.loopEnabled = function() {
		return this._loopEnabled;
	};

	Sprite_Balloon.prototype.speed = function() {
		return speed;
	};

	Sprite_Balloon.prototype.waitTime = function() {
		return waitTime;
	};

	const _Sprite_Balloon_updatePosition = Sprite_Balloon.prototype.updatePosition;
	Sprite_Balloon.prototype.updatePosition = function() {
		_Sprite_Balloon_updatePosition.call(this);
		this.x += this._offsetX;
		this.y += this._offsetY;
	};

	//-----------------------------------------------------------------------------
	// Spriteset_Map

	let destroy = false;
	const _Spriteset_Map_destroy = Spriteset_Map.prototype.destroy;
	Spriteset_Map.prototype.destroy = function(options) {
		destroy = true;
		_Spriteset_Map_destroy.call(this, options);
		destroy = false;
	};

	const _Spriteset_Map_initialize = Spriteset_Map.prototype.initialize;
	Spriteset_Map.prototype.initialize = function() {
		_Spriteset_Map_initialize.call(this);
		this.updateBalloons();
	};

	const _Spriteset_Map_createBalloon = Spriteset_Map.prototype.createBalloon;
	Spriteset_Map.prototype.createBalloon = function(request) {
		const targetSprite = this.findTargetSprite(request.target);
		let sprite = targetSprite && this._balloonSprites.find(sprite => sprite && sprite._target === targetSprite);
		if (sprite) {
			sprite.targetObject = request.target;
			sprite.setup(targetSprite, request.balloonId);
			this._effectsContainer.addChild(sprite);
		} else {
			_Spriteset_Map_createBalloon.call(this, request);
			sprite = targetSprite && this._balloonSprites.find(sprite => sprite && sprite._target === targetSprite);
		}
		if (sprite && sprite.setupBalloonPlus) {
			sprite.setupBalloonPlus(request);
		}
	};

	const _Spriteset_Map_removeBalloon = Spriteset_Map.prototype.removeBalloon;
	Spriteset_Map.prototype.removeBalloon = function(sprite) {
		_Spriteset_Map_removeBalloon.call(this, sprite);
		const targetObject = sprite.targetObject;
		if (!destroy) {
			if (targetObject.setLoopingBalloon) {
				targetObject.setLoopingBalloon(false);
			}
		}
		if (targetObject.setBalloonId && !targetObject.loopingBalloon()) {
			targetObject.setBalloonId(0);
		}
	};

	//-----------------------------------------------------------------------------
	// Game_Event

	const _Game_Event_setupPageSettings = Game_Event.prototype.setupPageSettings;
	Game_Event.prototype.setupPageSettings = function() {
		_Game_Event_setupPageSettings.call(this);
		this.setupSettingsBalloonPlus();
	};

	Game_Event.prototype.setupSettingsBalloonPlus = function() {
		const page = this.page();
		const lastBalloonId = this.symbolBalloonId();
		const balloonId = page.symbolBalloonId;
		if (!this._locked && balloonId !== lastBalloonId) {
			this.showBalloon(balloonId, false, balloonId > 0, true);
		}
		this.setSymbolBalloonId(balloonId);
	};

	const _Game_Event_lock = Game_Event.prototype.lock;
	Game_Event.prototype.lock = function() {
		const page = this.page();
		const balloonId = this.symbolBalloonId();
		if (!this._locked && balloonId > 0 && !page.symbolBalloonKeeping) {
			this.removeBalloon();
		}
		_Game_Event_lock.call(this);
	};

	const _Game_Event_unlock = Game_Event.prototype.unlock;
	Game_Event.prototype.unlock = function() {
		const balloonId = this.symbolBalloonId();
		if (this._locked && balloonId > 0 && (this.balloonId() !== balloonId || !this.loopingBalloon())) {
			this.showBalloon(balloonId, false, true, true);
		}
		_Game_Event_unlock.call(this);
	};

	//-----------------------------------------------------------------------------
	// DataManager

	const _DataManager_onLoad = DataManager.onLoad;
	DataManager.onLoad = function(object) {
		_DataManager_onLoad.call(this, object);
		if (this.isMapObject(object) && Array.isArray(object.events)) {
			for (const event of object.events) {
				if (event && event.pages){
					extractMetadata(event);
				}
			}
		}
	};

	function extractMetadata(data) {
		for (const page of data.pages) {
			const comment = findComment(page);
			const commentData = {"note": comment};
			DataManager.extractMetadata(commentData);
			addPageSettings(page, commentData.meta);
		}
	}

	function findComment(page) {
		const list = page.list;
		if (!list[0] && list[0].code !== 108) {
			return "";
		}
		let comment = list[0].parameters[0];
		for (let i = 1; list[i] && list[i].code === 408; i++) {
			comment += list[i].parameters[0];
		}
		return comment;
	}

	function addPageSettings(page, meta) {
		const data = (meta['symbolBalloon'] || "").split(",");
		page.symbolBalloonId = findBalloonId(data[0] || 0);
		page.symbolBalloonKeeping = data[1] === 'true';
	}

}