// =====================
// HeartDisplayPlugin.js
// =====================

/*:
 * @target MZ
 * @plugindesc A plugin that enables a Zelda-like HP system using hearts.
 * @author Sang Hendrix & ChatGPT
 * @url http://facebook.com/hendrixguy
 *
 * @param showHearts
 * @text Show hearts on start?
 * @type boolean
 * @desc Choose whether or not the hearts are enabled by default.
 * @default true
 * 
 * @param heartImage
 * @text Heart image file
 * @type file
 * @dir img/
 * @desc Choose your heart image file.
 * 
 * @param BreakHead
 * @text --------------------------
 * @default ----------------------------------
 * 
 * @param showHeartBlink
 * @text Low HP Blink Effect
 * @type boolean
 * @desc Choose whether or not the hearts should blink red when the current Hearts is low.
 * @default true
 * 
 * @param blinkSpeed
 * @text Blink speed
 * @type number
 * @desc Set blink speed of the Blink effect. The higher the number, the slower it blinks.
 * @default 30
 * 
 * @param blinkThreshold
 * @text Blink Threshold
 * @type select
 * @desc Blood threshold for the blinking effect to begin. Choose the ratio between current blood and maximum blood.
 * @default 1/4
 * @option 1/5
 * @value 1/5
 * @option 1/4
 * @value 1/4
 * @option 1/3
 * @value 1/3
 * @option 1/2
 * @value 1/2
 * 
 * @param BreakSettings
 * @text --------------------------
 * @default ----------------------------------
 * 
 * @param commonEventId
 * @text Common Event ID
 * @type common_event
 * @desc Choose an common event ID to be triggered when the player's out of hearts.
 * @default 1
 * 
 * @param BreakCommonEvent
 * @text --------------------------
 * @default ----------------------------------
 * 
 * @param currentHeartsVariable
 * @text Current Hearts Variable
 * @type variable
 * @desc Choose a variable to store the current hearts value.
 * @default 2
 *
 * @param maxHearts
 * @text Maximum Hearts
 * @type number
 * @desc The default maximum hearts to show when the game begins.
 * @default 5
 * 
 * @param BreakMax
 * @text --------------------------
 * @default ----------------------------------
 *
 * @param positionX
 * @text X position
 * @type number
 * @desc Choose X position of the first Heart.
 * @default 8
 * 
 * @param positionY
 * @text Y Position
 * @type number
 * @desc Choose Y position of the first Heart.
 * @default 8
 * 
 * @param heartsPerRow
 * @text Hearts per Row
 * @type number
 * @desc Set the limit of hearts can be shown per row.
 * @default 8
 *
 * @param rowSpacing
 * @text Row Spacing
 * @type number
 * @desc Set the spacing between heart rows.
 * @default 40
 *
 * @param heartSpacing
 * @text Heart Spacing
 * @type number
 * @desc Set the spacing between each heart.
 * @default 8
 * 
 * @help
 * Version 1.0.0
 * This plugin will help you create a Zelda-like HP system using hearts.
 * Contact me if you want to get help with the plugin: 
 * http://facebook.com/hendrixguy
 * 
 * FEATURES:
 * - Hearts blink when the amount of current HP is low
 * - Change maximum hearts during play
 * - Increase and decrease hearts
 * - Hearts can be hidden or shown during play
 * - Can set hearts per row and spacing
 * - Can automatically trigger an common event ID when run out of hearts
 * - Check Plugin Command for more.
 *
 * @command setHeartVisibility
 * @text Set Heart Visibility
 * @desc Set the visibility of the hearts on the screen.
 * @arg visible
 * @type boolean
 * @text Visible
 * @desc Set the visibility of the hearts.
 * @default true
 * 
 * @command increaseHeart
 * @text Increase Heart
 * @desc Increase the current number of hearts by 1.
 *
 * @command decreaseHeart
 * @text Decrease Heart
 * @desc Decrease the current number of hearts by 1.
 * 
 * @command setMaxHearts
 * @text Change Max Hearts
 * @desc Change the number of Max Hearts
 *
 * @arg newMaxHearts
 * @text New Max Hearts
 * @type number
 * @desc Change the number of Max Hearts
 * @default 5
*/
(() => {
	const parameters = PluginManager.parameters("HeartDisplayPlugin");
	const heartImagePath = parameters["heartImage"];
	const showHeartBlink = JSON.parse(parameters["showHeartBlink"]);
	const currentHeartsVariable = parseInt(parameters["currentHeartsVariable"]);
	let maxHearts = parseInt(parameters["maxHearts"]);
	const positionX = parseInt(parameters["positionX"]);
	const positionY = parseInt(parameters["positionY"]);
	const heartsPerRow = parseInt(parameters["heartsPerRow"]);
	const rowSpacing = parseInt(parameters["rowSpacing"]);
	const heartSpacing = parseInt(parameters["heartSpacing"]);
	const showHearts = JSON.parse(parameters["showHearts"]);

	let heartVisible = showHearts;

	const _createDisplayObjects = Scene_Map.prototype.createDisplayObjects;
	const _Game_System_initialize = Game_System.prototype.initialize;
	let globalCurrentHearts = maxHearts;

	Game_System.prototype.initialize = function () {
		_Game_System_initialize.call(this);
		const currentHearts = globalCurrentHearts;
		if (typeof currentHearts !== 'number' || currentHearts < 0 || currentHearts > maxHearts) {
			globalCurrentHearts = maxHearts;
			this._currentHearts = maxHearts;
		} else {
			this._currentHearts = currentHearts;
		}
		console.log('Khởi tạo. Máu hiện tại =', this._currentHearts);
		console.log('Khởi tạo. Máu tối đa =', maxHearts);
		if (heartDisplay) {
			heartDisplay.refresh();
		}
	};

	const createDisplayObjects = Scene_Map.prototype.createDisplayObjects;
	Scene_Map.prototype.createDisplayObjects = function () {
		createDisplayObjects.call(this);
		if (showHearts) {
			$gameVariables.setValue(currentHeartsVariable, maxHearts);
			heartDisplay = new HeartDisplay(maxHearts);
			heartDisplay.setHeartVisibility(heartVisible);
			this.addChild(heartDisplay);
		}
	};


	class HeartDisplay extends Sprite {
		constructor(maxHearts) {
			super();
			this._maxHearts = maxHearts;
			this._heartSprites = [];
			this.loadHeartImage();
			this._previousHearts = -1;
		}

		refresh() {
			this.updateHeartsDisplay();
		}


		updateHeartsDisplay() {
			if (!heartVisible) return;
			const currentHearts = $gameSystem._currentHearts;
			const blinkThreshold = parameters["blinkThreshold"];
			const thresholdHearts = this._maxHearts * (blinkThreshold === "1/2" ? 0.5 : blinkThreshold === "1/3" ? 1 / 3 : blinkThreshold === "1/4" ? 0.25 : 0.2);
			const shouldBlink = showHeartBlink && currentHearts <= thresholdHearts;
			const blinkSpeed = parseInt(parameters["blinkSpeed"]);
			const justReachedZero = this._previousHearts !== 0 && currentHearts === 0;
			const justIncreased = this._previousHearts !== -1 && this._previousHearts < currentHearts;
			const justDecreased = this._previousHearts !== -1 && this._previousHearts > currentHearts;

			for (let i = 0; i < this._maxHearts; i++) {
				if (this._heartSprites[i]) {
					const blinkThisHeart = shouldBlink && i < currentHearts;

					if (i < currentHearts) {
						if  (blinkThisHeart) {
							const blinkValue = 128 + Math.round(Math.sin(Graphics.frameCount / blinkSpeed) * 64);
							this._heartSprites[i].setBlendColor([255 * (1 - blinkValue / 255), 0, 0, blinkValue]);
							this._heartSprites[i].opacity = 255;
						} else {
							this._heartSprites[i].setBlendColor([0, 0, 0, 0]);
							this._heartSprites[i].opacity = 255;
						}


					} else {

						if (this._previousHearts < currentHearts && i === this._previousHearts) {
							this._heartTintFrames[i] = 0;
						} else if (this._previousHearts > currentHearts && i === currentHearts) {
							this._heartTintFrames[i] = 0;
						} else if (justIncreased && i === currentHearts - 1) {
							this._heartTintFrames[i] = 0;
						} else {
							this._heartTintFrames[i] = this._heartTintFrames[i] + 1;
						}
						// Fade từ màu đen sang màu gốc trong 15 frame
						if (justIncreased && i === currentHearts - 1) {
							if (this._heartTintFrames[i] < 15) {
								this._heartSprites[i].setBlendColor([0, 255, 0, greenValue]);
								this._heartSprites[i].opacity = 255;
							} else {
								this._heartSprites[i].setBlendColor([0, 255, 0, 0]);
								this._heartSprites[i].opacity = 255;
							}
						} else if (this._heartTintFrames[i] < 14) {
							this._heartSprites[i].setBlendColor([255, 0, 0, 255 * (1 - this._heartTintFrames[i] / 15)]);
						} else {
							this._heartSprites[i].setBlendColor([0, 0, 0, 255 * ((this._heartTintFrames[i] - 7) / 10)]);
							this._heartSprites[i].opacity = 255;
						}
					}
				}
			}
			if (currentHearts === 0 && this._previousHearts !== 0 && this._previousHearts !== -1) {
				const commonEventId = parseInt(parameters["commonEventId"]);
				$gameTemp.reserveCommonEvent(commonEventId);
			}
			this._previousHearts = currentHearts;

		}
		setHeartTint(index, tint) {
			if (index < 0 || index >= this._maxHearts) {
				return;
			}
	
			if (tint === "black") {
				this._heartSprites[index].setColorTone([-255, -255, -255, 255]);
				this._heartStates[index] = 0;
			} else {
				this._heartSprites[index].setColorTone([0, 0, 0, 0]);
				this._heartStates[index] = 1;
			}
		}

		update() {
			super.update();
			this.updateHeartsDisplay();
			this._heartSprites.forEach((heartSprite) => {
				heartSprite.visible = heartVisible;
			});
		}

		updateMaxHearts(newMaxHearts) {
			this._maxHearts = newMaxHearts;
			while (this._heartSprites.length > newMaxHearts) {
				this.removeChild(this._heartSprites.pop());
			}
			while (this._heartSprites.length < newMaxHearts) {
				const i = this._heartSprites.length;
				const row = Math.floor(i / heartsPerRow);
				const col = i % heartsPerRow;
		
				const backgroundSprite = new Sprite();
				backgroundSprite.bitmap = new Bitmap(this._heartImage.width, this._heartImage.height);
				backgroundSprite.bitmap.fillRect(0, 0, this._heartImage.width, this._heartImage.height, "rgba(0, 0, 0, 0)");
				backgroundSprite.x = positionX + col * (backgroundSprite.width + heartSpacing);
				backgroundSprite.y = positionY + row * rowSpacing;
				this.addChild(backgroundSprite);
		
				const heartSprite = new Sprite();
				heartSprite.bitmap = this._heartImage;
				heartSprite.x = positionX + col * (heartSprite.width + heartSpacing);
				heartSprite.y = positionY + row * rowSpacing;
				this.addChild(heartSprite);
				this._heartSprites.push(heartSprite);
			}
		}
		

		setHeartVisibility(visible) {
			heartVisible = visible;
    for (let i = 0; i < this._heartSprites.length; i++) {
        if (visible) {
            this._heartSprites[i].visible = this._heartStates[i] === 1;
        } else {
            this._heartSprites[i].visible = false;
        }}
		}

		loadHeartImage() {
			this._heartImage = ImageManager.loadBitmap("img/", heartImagePath);
			this._heartImage.addLoadListener(() => {
				this.createHeartSprites();
			});
		}

		createHeartSprites() {
			const HEARTS_PER_ROW = heartsPerRow;
			const ROW_SPACING = rowSpacing;
			this._heartTintFrames = new Array(this._maxHearts).fill(0);
			this._heartStates = new Array(this._maxHearts).fill(1);

			for (let i = 0; i < this._maxHearts; i++) {
				const row = Math.floor(i / HEARTS_PER_ROW);
				const col = i % HEARTS_PER_ROW;

				const backgroundSprite = new Sprite();
				backgroundSprite.bitmap = new Bitmap(this._heartImage.width, this._heartImage.height);
				backgroundSprite.bitmap.fillRect(0, 0, this._heartImage.width, this._heartImage.height, "rgba(0, 0, 0, 0)");
				backgroundSprite.x = positionX + col * (backgroundSprite.width + heartSpacing);
				backgroundSprite.y = positionY + row * ROW_SPACING;
				this.addChild(backgroundSprite);

				const heartSprite = new Sprite();
				heartSprite.bitmap = this._heartImage;
				heartSprite.x = positionX + col * (heartSprite.width + heartSpacing);
				heartSprite.y = positionY + row * ROW_SPACING;
				heartSprite.visible = heartVisible && this._heartStates[i] === 1;
				this.addChild(heartSprite);
				this._heartSprites.push(heartSprite);
			}
		}
	}
	let heartDisplay = null;


	PluginManager.registerCommand("HeartDisplayPlugin", "setHeartVisibility", args => {
		const visible = JSON.parse(args.visible);
		if (heartDisplay) {
			heartDisplay.setHeartVisibility(visible);
		}
	});

	PluginManager.registerCommand("HeartDisplayPlugin", "increaseHeart", () => {
		$gameSystem._currentHearts = Math.min($gameSystem._currentHearts + 1, maxHearts);
		$gameVariables.setValue(currentHeartsVariable, $gameSystem._currentHearts);
	});

	PluginManager.registerCommand("HeartDisplayPlugin", "decreaseHeart", () => {
		$gameSystem._currentHearts = Math.max($gameSystem._currentHearts - 1, 0);
		$gameVariables.setValue(currentHeartsVariable, $gameSystem._currentHearts);
	});

	PluginManager.registerCommand("HeartDisplayPlugin", "setMaxHearts", args => {
		const newMaxHearts = parseInt(args.newMaxHearts);
	
		if (!isNaN(newMaxHearts)) {
			maxHearts = newMaxHearts;
			if (heartDisplay) {
				heartDisplay.updateMaxHearts(newMaxHearts);
			}
		} else {
			console.error(`Invalid maxHearts value: ${args.newMaxHearts}`);
		}
	});
	

	const _Game_System_onBeforeSave = Game_System.prototype.onBeforeSave;
	Game_System.prototype.onBeforeSave = function () {
		_Game_System_onBeforeSave.call(this);
		$gameVariables.setValue(currentHeartsVariable, this._currentHearts);
	};

})();