// LayoutForWvga.js Ver.1.0.3
// MIT License (C) 2022 あわやまたな
// http://opensource.org/licenses/mit-license.php

/*:
* @target MZ
* @orderAfter FesStyleBattle
* @plugindesc レイアウトをWVGA（800x480）に最適化します。
* @author あわやまたな (Awaya_Matana)
* @url https://awaya3ji.seesaa.net/article/494379285.html
* @help 解像度が3DSの二倍、マップの表示範囲がツクールフェスと同等になります。
* ツクールフェス再現プラグインを複数使用する前提のプラグインです。
*
* 一部UIが崩壊するため、当該プラグインのパラメータを適宜調整して下さい。
* 特にメニュー、セーブ画面のプラグインはかなり影響を受けます。
*
* [更新履歴]
* 2022/12/03：Ver.1.0.0　公開。
* 2022/12/09：Ver.1.0.1　FesStyleBattleの判別方法を修正。
* 2023/01/22：Ver.1.0.2　gaugeLineHeightを修正。
* 2023/02/28：Ver.1.0.3　チャージタイムゲージを修正。
*
*/

'use strict';

{
	const fesStyleBattle = $plugins.some(plugin => Utils.extractFileName(plugin.name) === "FesStyleBattle" && plugin.status);
	const padding = 8;

	//-----------------------------------------------------------------------------
	// Scene_Boot

	const _Scene_Boot_resizeScreen = Scene_Boot.prototype.resizeScreen;
	Scene_Boot.prototype.resizeScreen = function() {
		const adv = $dataSystem.advanced;
		adv.screenWidth = adv.uiAreaWidth = 800;
		adv.screenHeight = adv.uiAreaHeight = 480;
		_Scene_Boot_resizeScreen.call(this);
	};

	//-----------------------------------------------------------------------------
	// Scene_MenuBase

	Scene_MenuBase.prototype.buttonAreaBottom = function() {
		return 0;
	};

	Scene_MenuBase.prototype.mainAreaHeight = function() {
		return Graphics.boxHeight - this.helpAreaHeight();
	};

	//-----------------------------------------------------------------------------
	// Scene_Name

	Scene_Name.prototype.editWindowRect = function() {
		const inputWindowHeight = this.calcWindowHeight(9, true);
		const padding = $gameSystem.windowPadding();
		const ww = 600;
		const wh = Graphics.boxHeight - inputWindowHeight;
		const wx = (Graphics.boxWidth - ww) / 2;
		const wy = 0;
		return new Rectangle(wx, wy, ww, wh);
	};

	Scene_Name.prototype.inputWindowRect = function() {
		const wx = this._editWindow.x;
		const wy = this._editWindow.y + this._editWindow.height;
		const ww = this._editWindow.width;
		const wh = this.calcWindowHeight(9, true);
		return new Rectangle(wx, wy, ww, wh);
	};

	//-----------------------------------------------------------------------------
	// Scene_Battle

	if (fesStyleBattle) {
		const _Scene_Battle_logWindowRect = Scene_Battle.prototype.logWindowRect;
		Scene_Battle.prototype.logWindowRect = function() {
			const rect = _Scene_Battle_logWindowRect.call(this);
			rect.height = Window_BattleLog.prototype.fittingHeight(Window_BattleLog.prototype.maxLines());
			rect.y = Graphics.boxHeight - rect.height;
			return rect;
		};
	}

	if (fesStyleBattle) {
		Window_BattleLog.prototype.lineHeight = function() {
			return 28;
		};

		Window_BattleLog.prototype.itemPadding = function() {
			return 4;
		};

		const _Window_BattleLog_resetFontSettings = Window_BattleLog.prototype.resetFontSettings;
		Window_BattleLog.prototype.resetFontSettings = function() {
			_Window_BattleLog_resetFontSettings.call(this);
			this.contents.fontSize = $gameSystem.mainFontSize()-2;
		};
	}

	//-----------------------------------------------------------------------------
	// Window_NameBox

	Window_NameBox.prototype.lineHeight = function() {
		return 32;
	};

	//-----------------------------------------------------------------------------
	// Window_Base

	const _Window_Base_itemPadding = Window_Base.prototype.itemPadding;
	Window_Base.prototype.itemPadding = function() {
		return _Window_Base_itemPadding.call(this) + _Game_System_windowPadding.call($gameSystem) - padding;
	};

	//-----------------------------------------------------------------------------
	// Window_Selectable

	Window_Selectable.prototype.itemHeight = function() {
		return Window_Scrollable.prototype.itemHeight.call(this);
	};

	//-----------------------------------------------------------------------------
	// Game_System

	const _Game_System_windowPadding = Game_System.prototype.windowPadding;
	Game_System.prototype.windowPadding = function() {
		return padding;
	};

	//-----------------------------------------------------------------------------
	// Scene_Message

	const _Scene_Message_messageWindowRect = Scene_Message.prototype.messageWindowRect;
	Scene_Message.prototype.messageWindowRect = function() {
		const rect = _Scene_Message_messageWindowRect.call(this);
		const pad = $gameSystem.windowPadding() - _Game_System_windowPadding.call($gameSystem);
		rect.height += padding;
		rect.width -= padding;
		return rect;
	};

	//-----------------------------------------------------------------------------
	// Window_Message

	const _Window_Message_newPage = Window_Message.prototype.newPage;
	Window_Message.prototype.newPage = function(textState) {
		_Window_Message_newPage.call(this, textState);
		const pad = $gameSystem.windowPadding() - _Game_System_windowPadding.call($gameSystem);
		textState.y -= pad;
	};

	const _Window_Message_newLineX = Window_Message.prototype.newLineX;
	Window_Message.prototype.newLineX = function(textState) {
		const newLineX = _Window_Message_newLineX.call(this, textState);
		//const padding = padding;
		return textState.rtl ? newLineX - padding : newLineX + padding;
	};

	//-----------------------------------------------------------------------------
	// Window_NameEdit

	Window_NameEdit.prototype.refresh = function() {
		this.contents.clear();
		const y = Math.floor((this.innerHeight - ImageManager.faceHeight)/2);
		this.drawActorFace(this._actor, 0, y);
		for (let i = 0; i < this._maxLength; i++) {
			this.drawUnderline(i);
		}
		for (let j = 0; j < this._name.length; j++) {
			this.drawChar(j);
		}
		const rect = this.itemRect(this._index);
		this.setCursorRect(rect.x, rect.y, rect.width, rect.height);
	};

	const _Window_NameEdit_itemRect = Window_NameEdit.prototype.itemRect;
	Window_NameEdit.prototype.itemRect = function(index) {
		const rect = _Window_NameEdit_itemRect.call(this, index);
		rect.y = (this.innerRect.height - this.lineHeight()) /2;
		return rect;
	};

	Window_NameEdit.prototype.updatePadding = function() {
		this.padding = 6;
	};

	//-----------------------------------------------------------------------------
	// Sprite_Gauge

	Sprite_Gauge.prototype.bitmapHeight = function() {
		return 20;
	};

	Sprite_Gauge.prototype.textHeight = function() {
		return 16;
	};

	Sprite_Gauge.prototype.gaugeHeight = function() {
		return 6;
	};

	Sprite_Gauge.prototype.gaugeX = function() {
		return 0;
	};

	Sprite_Gauge.prototype.labelY = function() {
		return -1;
	};

	Sprite_Gauge.prototype.labelFontSize = function() {
		return 16;
	};

	Sprite_Gauge.prototype.valueFontSize = function() {
		return 16;
	};

	const _Sprite_Gauge_drawGaugeRect = Sprite_Gauge.prototype.drawGaugeRect;
	Sprite_Gauge.prototype.drawGaugeRect = function(x, y, width, height) {
		if (this.label()) y = this.textHeight() - 2;
		_Sprite_Gauge_drawGaugeRect.call(this, x, y, width, height);
	};
	
	//-----------------------------------------------------------------------------
	// Window_StatusBase

	Window_StatusBase.prototype.gaugeLineHeight = function() {
		return 22;
	};

	//-----------------------------------------------------------------------------
	// Window_EquipStatus

	Window_EquipStatus.prototype.paramY = function(index) {
		return this.faceHeight() + Math.floor(this.lineHeight() * (index + 1));
	};

	Window_EquipStatus.prototype.refresh = function() {
		this.contents.clear();
		if (this._actor) {
			const nameRect = this.itemLineRect(0);
			this.drawActorName(this._actor, nameRect.x, 0, nameRect.width);
			this.drawActorFace(this._actor, nameRect.x, nameRect.height, ImageManager.faceWidth, this.faceHeight());
			this.drawAllParams();
		}
	};

	Window_EquipStatus.prototype.faceHeight = function() {
		return Math.floor(this.lineHeight()*3);
	};

	//-----------------------------------------------------------------------------
	// Sprite_Button

	Sprite_Button.prototype.updateOpacity = function() {
		this.opacity = this._pressed ? 224 : 160;
	};

}