//===========================================================================
// Open Digital World - Custom Font Message Plugin v1.2.0
//===========================================================================

/*:
 * @target MZ
 * @plugindesc [v1.2.0] - Manage custom fonts in game messages.
 * @author Open Digital World
 * @url https://opendigitalworld.itch.io/rmmz-custom-font-message-plugin
 * @base GabeMZ_CustomFontLoader
 *
 * @help
 *------------------------------------------------------------------------------
 * Open Digital World - Custom Font Message Plugin
 *------------------------------------------------------------------------------
 *
 * This plugin allows you to use custom fonts to your game and use them to
 * change the appearance of text in messages displayed.
 *
 * This plugin is an add-on of the plugin "Gabe MZ - Custom Font Loader" which
 * you can get here:
 *
 * https://github.com/comuns-rpgmaker/GabeMZ/blob/master/GabeMZ_CustomFontLoader.js
 *
 *------------------------------------------------------------------------------
 * How to use it?
 *------------------------------------------------------------------------------
 *
 * 1. Put your custom fonts in the /fonts folder of the game project.
 *
 * 2. Configure the plugin "Gabe MZ - Custom Font Loader".
 *
 * 3. Use the following plugin commands in your game before calling the commands
 *    "Show Text", "Show Choices" or "Scroll Text" in an event:
 *      - <Reset Custom Font>: Restore the default font settings.
 *      - <Use Custom Font>: Replace the default font settings with your own.
 *          - <Font Name>: the name you used to declare the font.
 *          - <Font Size>: the size of the text.
 *          - <Font Color>: the color of the text.
 *          - <Font Outline Color>: the color of the text outline.
 *          - <Font Outline Width>: the width of the text outline.
 *      - <Use Custom Font Name>: Replace the default font by one of your
 *        custom fonts.
 *          - <Font Name>: the name you used to declare the font.
 *          - <Reset Before Change>: the command to reset the custom font
 *            settings before applying this change.
 *      - <Use Custom Font Size>: Replace the default font size.
 *          - <Font Size>: the size of the text.
 *          - <Reset Before Change>: the command to reset the custom font
 *            settings before applying this change.
 *      - <Use Custom Font Color>: Replace the default font color.
 *          - <Font Color>: the color of the text.
 *          - <Reset Before Change>: the command to reset the custom font
 *            settings before applying this change.
 *      - <Use Custom Font Outline>: Replace the default font outline color
 *        and size.
 *          - <Font Outline Color>: the color of the text outline.
 *          - <Font Outline Width>: the width of the text outline.
 *          - <Reset Before Change>: the command to reset the custom font
 *            settings before applying this change.
 *
 *------------------------------------------------------------------------------
 * Terms of Use - License MIT
 *------------------------------------------------------------------------------
 *
 * Copyright (c) 2021 Open Digital World / Public Productions
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 *------------------------------------------------------------------------------
 * Version history
 *------------------------------------------------------------------------------
 *
 * 23.09.2021 v1.0.0 - Initial release.
 * 26.09.2021 v1.1.0 - Added support for choices and scroll text.
 * 05.10.2021 v1.2.0 - Added support for text color and outline. Added new
 *                     plugin commands to specifically change the font, size,
 *                     color or outline of text.
 *
 *------------------------------------------------------------------------------
 * Overrides of core functions
 *------------------------------------------------------------------------------
 *
 * New functions:
 *   - Window_Message.prototype.resetFontSettings
 *   - Window_ChoiceList.prototype.resetFontSettings
 *   - Window_ScrollText.prototype.resetFontSettings
 *
 *------------------------------------------------------------------------------
 * Known incompatibilities with other plugins
 *------------------------------------------------------------------------------
 *
 * None.
 *
 *------------------------------------------------------------------------------
 *
 *
 * @command resetCustomFont
 * @text Reset Custom Font
 * @desc Restore the default font settings.
 *
 * @command useCustomFont
 * @text Use Custom Font
 * @desc Replace the default font settings with your own.
 *
 * @arg fontName
 * @text Font Name
 * @desc The name you used to declare the font.
 * @type string
 *
 * @arg fontSize
 * @text Font Size
 * @desc The size of the text. A value between 6 and 108.
 * @type number
 * @min 6
 * @max 108
 * @default 20
 *
 * @arg fontColor
 * @text Font Color
 * @desc The color of the text. Can be #HEX, rgba() or one of the 0-31 color codes from the windowskin.
 * @type string
 * @default 0
 *
 * @arg fontOutlineColor
 * @text Font Outline Color
 * @desc The color of the text outline. Can be #HEX, rgba() or one of the 0-31 color codes from the windowskin.
 * @type string
 * @default rgba(0, 0, 0, 0.6)
 *
 * @arg fontOutlineWidth
 * @text Font Outline Width
 * @desc The width of the text outline. A value between 0 and 12.
 * @type number
 * @min 0
 * @max 12
 * @default 0
 *
 * @command useCustomFontName
 * @text Use Custom Font Name
 * @desc Replace the default font by one of your custom fonts.
 *
 * @arg fontName
 * @text Font Name
 * @desc The name you used to declare the font.
 * @type string
 *
 * @arg resetBeforeChange
 * @text Reset Before Change
 * @desc The command to reset the custom font settings before applying this change.
 * @type boolean
 * @default true
 *
 * @command useCustomFontSize
 * @text Use Custom Font Size
 * @desc Replace the default font size.
 *
 * @arg fontSize
 * @text Font Size
 * @desc The size of the text. A value between 6 and 108.
 * @type number
 * @min 6
 * @max 108
 * @default 20
 *
 * @arg resetBeforeChange
 * @text Reset Before Change
 * @desc The command to reset the custom font settings before applying this change.
 * @type boolean
 * @default true
 *
 * @command useCustomFontColor
 * @text Use Custom Font Color
 * @desc Replace the default font color.
 *
 * @arg fontColor
 * @text Font Color
 * @desc The color of the text. Can be #HEX, rgba() or one of the 0-31 color codes from the windowskin.
 * @type string
 * @default 0
 *
 * @arg resetBeforeChange
 * @text Reset Before Change
 * @desc The command to reset the custom font settings before applying this change.
 * @type boolean
 * @default true
 *
 * @command useCustomFontOutline
 * @text Use Custom Font Outline
 * @desc Replace the default font outline color and size.
 *
 * @arg fontOutlineColor
 * @text Font Outline Color
 * @desc The color of the text outline. Can be #HEX, rgba() or one of the 0-31 color codes from the windowskin.
 * @type string
 * @default rgba(0, 0, 0, 0.6)
 *
 * @arg fontOutlineWidth
 * @text Font Outline Width
 * @desc The width of the text outline. A value between 0 and 12.
 * @type number
 * @min 0
 * @max 12
 * @default 0
 *
 * @arg resetBeforeChange
 * @text Reset Before Change
 * @desc The command to reset the custom font settings before applying this change.
 * @type boolean
 * @default true
 *
 */

var Imported = Imported || {};
Imported.ODW_CustomFontMessage = true;

var ODW = ODW || {};
ODW.CFM = ODW.CFM || {};
ODW.CFM.pluginName = "ODW_CustomFontMessage";
ODW.CFM.pluginVersion = [1, 2, 0];

(($) => {

	'use strict';

	/*
	 *------------------------------------------------------------------------------
	 * CUSTOM FONT
	 *------------------------------------------------------------------------------
	 */

	// Declare the current custom font name.
	$._customFontName = false;

	// Declare the current custom font size.
	$._customFontSize = false;

	// Declare the current custom font color.
	$._customFontColor = false;

	// Declare the current custom font outline color.
	$._customFontOutlineColor = false;

	// Declare the current custom font outline width.
	$._customFontOutlineWidth = false;

	/*
	 * Sanitize an integer value.
	 *
	 * @param string The value to sanitize
	 *
	 * @return integer
	 */
	$.sanitizeInteger = function(value, min, max) {
		if (value) {
			value = parseInt(value);
			if (min <= value && value <= max) {
				return value;
			}
		}
		return false;
	}

	/*
	 * Sanitize a color value.
	 *
	 * @param string The value to sanitize
	 *
	 * @return string
	 */
	$.sanitizeColor = function(value) {
		if (value) {
			let pattern = /#[0-9ABCDEF]{6}/i;
			if (value.match(pattern)) {
				return value;
			}
			pattern = /rgba\(\d+,\s?\d+,\s?\d+,\s?\d+.?\d*\)$/;
			if (value.match(pattern)) {
				return value;
			}
			pattern = parseInt(value);
			if (0 <= pattern && pattern <= 31) {
				return ColorManager.textColor(value);
			}
		}
		return false;
	}

	/*
	 * Sanitize a boolean value.
	 *
	 * @param string The value to sanitize
	 *
	 * @return boolean
	 */
	$.sanitizeBoolean = function(value) {
		if (value === "true") {
			return true;
		} else {
			return false;
		}
	}

	/*
	 * Reset the custom font used.
	 *
	 * @return void
	 */
	$.resetCustomFont = function() {
		$._customFontName         = false;
		$._customFontSize         = false;
		$._customFontColor        = false;
		$._customFontOutlineColor = false;
		$._customFontOutlineWidth = false;
	}

	/*
	 * Use a custom font.
	 *
	 * @return void
	 */
	$.useCustomFont = function(fontName, fontSize, fontColor, fontOutlineColor, fontOutlineWidth) {
		$.resetCustomFont();
		$.useCustomFontName(fontName, false);
		$.useCustomFontSize(fontSize, false);
		$.useCustomFontColor(fontColor, false);
		$.useCustomFontOutline(fontOutlineColor, fontOutlineWidth, false);
	}

	/*
	 * Use a custom font name.
	 *
	 * @param string  The font name
	 * @param boolean The command to reset the current font settings
	 *
	 * @return void
	 */
	$.useCustomFontName = function(fontName, resetBeforeChange) {
		if (resetBeforeChange) {
			$.resetCustomFont();
		} else {
			$._customFontName = false;
		}
		if (Imported.GMZ_CustomFontLoader === true) {
			const customFonts = GabeMZ.CustomFontLoader.fonts;
			if (customFonts.length > 0) {
				for (let customFont of customFonts) {
					if (fontName === customFont.name) {
						$._customFontName = customFont.name;
						break;
					}
				}
				if ($._customFontName === false ){
					$.logErrorCustomFont(fontName, "No custom font configured for this font name.");
				}
			} else {
				$.logError("No custom fonts configured in the GabeMZ_CustomFontLoader plugin.");
			}
		} else {
			$.logError("Requires the GabeMZ_CustomFontLoader plugin.");
		}
	}

	/*
	 * Use a custom font size.
	 *
	 * @param integer The font size
	 * @param boolean The command to reset the current font settings
	 *
	 * @return void
	 */
	$.useCustomFontSize = function(fontSize, resetBeforeChange) {
		if (resetBeforeChange) {
			$.resetCustomFont();
		}
		$._customFontSize = fontSize;
	}

	/*
	 * Use a custom font color.
	 *
	 * @param string  The font color
	 * @param boolean The command to reset the current font settings
	 *
	 * @return void
	 */
	$.useCustomFontColor = function(fontColor, resetBeforeChange) {
		if (resetBeforeChange) {
			$.resetCustomFont();
		}
		$._customFontColor = fontColor;
	}

	/*
	 * Use a custom font outline.
	 *
	 * @param string  The font outline color
	 * @param integer The font outline width
	 * @param boolean The command to reset the current font settings
	 *
	 * @return void
	 */
	$.useCustomFontOutline = function(fontOutlineColor, fontOutlineWidth, resetBeforeChange) {
		if (resetBeforeChange) {
			$.resetCustomFont();
		}
		$._customFontOutlineColor = fontOutlineColor;
		$._customFontOutlineWidth = fontOutlineWidth;
	}

	/*
	 * Retrieve the custom font name used.
	 *
	 * @return string|false
	 */
	$.getCustomFontName = function() {
		return $._customFontName;
	}

	/*
	 * Retrieve the custom font size used.
	 *
	 * @return integer|false
	 */
	$.getCustomFontSize = function() {
		return $._customFontSize;
	}

	/*
	 * Retrieve the custom font color used.
	 *
	 * @return string|false
	 */
	$.getCustomFontColor = function() {
		return $._customFontColor;
	}

	/*
	 * Retrieve the custom font outline color used.
	 *
	 * @return string|false
	 */
	$.getCustomFontOutlineColor = function() {
		return $._customFontOutlineColor;
	}

	/*
	 * Retrieve the custom font outline width used.
	 *
	 * @return integer|false
	 */
	$.getCustomFontOutllineWidth = function() {
		return $._customFontOutlineWidth;
	}

	/*
	 *------------------------------------------------------------------------------
	 * LOG
	 *------------------------------------------------------------------------------
	 */

	/*
	 * Log the errors while processing the plugin in general.
	 *
	 * @param string The error text
	 *
	 * @return void
	 */
	$.logError = function(error) {
		console.log("Plugin: " + $.pluginName + "\nError: " + error);
	};

	/*
	 * Log the errors while processing a custom font.
	 *
	 * @param string The font name that caused the error
	 * @param string The error text
	 *
	 * @return void
	 */
	$.logErrorCustomFont = function(fontName, error) {
		console.log("Plugin: " + $.pluginName + "\nFont Name: " + fontName + "\nError: " + error);
	};

})(ODW.CFM);

//===========================================================================
// PluginManager
//===========================================================================

PluginManager.registerCommand(ODW.CFM.pluginName, "resetCustomFont", args => {
	ODW.CFM.resetCustomFont();
});

PluginManager.registerCommand(ODW.CFM.pluginName, "useCustomFont", args => {
	const fontName         = args.fontName;
	const fontSize         = ODW.CFM.sanitizeInteger(args.fontSize, 6, 108);
	const fontColor        = ODW.CFM.sanitizeColor(args.fontColor);
	const fontOutlineColor = ODW.CFM.sanitizeColor(args.fontOutlineColor);
	const fontOutlineWidth = ODW.CFM.sanitizeInteger(args.fontOutlineWidth, 0, 12);
	ODW.CFM.useCustomFont(fontName, fontSize, fontColor, fontOutlineColor, fontOutlineWidth);
});

PluginManager.registerCommand(ODW.CFM.pluginName, "useCustomFontName", args => {
	const fontName          = args.fontName;
	const resetBeforeChange = ODW.CFM.sanitizeBoolean(args.resetBeforeChange);
	ODW.CFM.useCustomFontName(fontName, resetBeforeChange);
});

PluginManager.registerCommand(ODW.CFM.pluginName, "useCustomFontSize", args => {
	const fontSize          = ODW.CFM.sanitizeInteger(args.fontSize, 6, 108);
	const resetBeforeChange = ODW.CFM.sanitizeBoolean(args.resetBeforeChange);
	ODW.CFM.useCustomFontSize(fontSize, resetBeforeChange);
});

PluginManager.registerCommand(ODW.CFM.pluginName, "useCustomFontColor", args => {
	const fontColor         = ODW.CFM.sanitizeColor(args.fontColor);
	const resetBeforeChange = ODW.CFM.sanitizeBoolean(args.resetBeforeChange);

	ODW.CFM.useCustomFontColor(fontColor, resetBeforeChange);
});

PluginManager.registerCommand(ODW.CFM.pluginName, "useCustomFontOutline", args => {
	const fontOutlineColor  = ODW.CFM.sanitizeColor(args.fontOutlineColor);
	const fontOutlineWidth  = ODW.CFM.sanitizeInteger(args.fontOutlineWidth, 0, 12);
	const resetBeforeChange = ODW.CFM.sanitizeBoolean(args.resetBeforeChange);
	ODW.CFM.useCustomFontOutline(fontOutlineColor, fontOutlineWidth, resetBeforeChange);
});

//===========================================================================
// Window_Message
//===========================================================================

// New function.
Window_Message.prototype.resetFontSettings = function() {
	Window_Base.prototype.resetFontSettings.call(this);
	const customFontName         = ODW.CFM.getCustomFontName();
	const customFontSize         = ODW.CFM.getCustomFontSize();
	const customFontColor        = ODW.CFM.getCustomFontColor();
	const customFontOutlineColor = ODW.CFM.getCustomFontOutlineColor();
	const customFontOutlineWidth = ODW.CFM.getCustomFontOutllineWidth();
	if (customFontName !== false) {
		this.contents.fontFace = customFontName + ", " + this.contents.fontFace;
	}
	if (customFontSize !== false) {
		this.contents.fontSize = customFontSize;
	}
	if (customFontColor !== false) {
		this.contents.textColor = customFontColor;
	}
	if (customFontOutlineColor !== false) {
		this.contents.outlineColor = customFontOutlineColor;
	}
	if (customFontOutlineWidth !== false) {
		this.contents.outlineWidth = customFontOutlineWidth;
	} else {
		this.contents.outlineWidth = 3;
	}
};

//===========================================================================
// Window_ChoiceList
//===========================================================================

// New function.
Window_ChoiceList.prototype.resetFontSettings = function() {
	Window_Base.prototype.resetFontSettings.call(this);
	const customFontName         = ODW.CFM.getCustomFontName();
	const customFontSize         = ODW.CFM.getCustomFontSize();
	const customFontColor        = ODW.CFM.getCustomFontColor();
	const customFontOutlineColor = ODW.CFM.getCustomFontOutlineColor();
	const customFontOutlineWidth = ODW.CFM.getCustomFontOutllineWidth();
	if (customFontName !== false) {
		this.contents.fontFace = customFontName + ", " + this.contents.fontFace;
	}
	if (customFontSize !== false) {
		this.contents.fontSize = customFontSize;
	}
	if (customFontColor !== false) {
		this.contents.textColor = customFontColor;
	}
	if (customFontOutlineColor !== false) {
		this.contents.outlineColor = customFontOutlineColor;
	}
	if (customFontOutlineWidth !== false) {
		this.contents.outlineWidth = customFontOutlineWidth;
	} else {
		this.contents.outlineWidth = 3;
	}
};

//===========================================================================
// Window_ScrollText
//===========================================================================

// New function.
Window_ScrollText.prototype.resetFontSettings = function() {
	Window_Base.prototype.resetFontSettings.call(this);
	const customFontName         = ODW.CFM.getCustomFontName();
	const customFontSize         = ODW.CFM.getCustomFontSize();
	const customFontColor        = ODW.CFM.getCustomFontColor();
	const customFontOutlineColor = ODW.CFM.getCustomFontOutlineColor();
	const customFontOutlineWidth = ODW.CFM.getCustomFontOutllineWidth();
	if (customFontName !== false) {
		this.contents.fontFace = customFontName + ", " + this.contents.fontFace;
	}
	if (customFontSize !== false) {
		this.contents.fontSize = customFontSize;
	}
	if (customFontColor !== false) {
		this.contents.textColor = customFontColor;
	}
	if (customFontOutlineColor !== false) {
		this.contents.outlineColor = customFontOutlineColor;
	}
	if (customFontOutlineWidth !== false) {
		this.contents.outlineWidth = customFontOutlineWidth;
	} else {
		this.contents.outlineWidth = 3;
	}
};
